# upload script based on https://developers.zenodo.org/?python#quickstart-upload
import os
import logging
import requests
import pathlib
import json
import argparse
from openscm_zenodo import zenodo


logger = logging.getLogger(__name__)
logging.basicConfig(filename='register_doi.log',
                    encoding='utf-8', level=logging.DEBUG)


class ZenodoAPI:

    def __init__(self, url: str, token: str, ssl_verify=False):
        self._url = url
        self._token = token
        self._ssl_verify = ssl_verify

        self._default_params = {'params': {'access_token': self._token},
                                'verify': self._ssl_verify,
                                'headers': {"Content-Type": "application/json"}}

        self.check_connection()

        logging.debug(f"Initialized ZenodoAPI to connect to {self._url}")

    def check_connection(self) -> None:
        """
        Check if connection to server can be established
        """
        r = requests.get(f'https://{self._url}/api/deposit/depositions',
                         params={'access_token': self._token}, verify=self._ssl_verify)

        if r.status_code != 200:
            raise ValueError(f'Could not establish connection to {self._url}')

    def register_doi(self, filename: str | pathlib.Path, version: str = None, deposition_id: str = None) -> str:
        """
        Upload file to Zenodo server and publish it to get a DOI
        """

        if deposition_id is None:
            r = requests.post(f'https://{self._url}/api/deposit/depositions',
                              **self._default_params)
            deposition_id = r.json()['id']

            logging.warning(
                f'Deposition_id is not provided. Creating a new deposit with id {deposition_id}')

        # # fetch bucket id of deposition
        # r = requests.get(f'https://{self._url}/api/deposit/depositions/{deposition_id}',
        #                  **self._default_params)
        # r.raise_for_status()

        # get latest version of the deposition
        try:
            r = requests.get(
                f'https://{self._url}/api/records/{deposition_id}/versions/latest', **self._default_params)
            r.raise_for_status()
            latest_deposition_id = r.json()['id']

        except KeyError:
            latest_deposition_draft_id = latest_deposition_id
        else:

            # create new version based on latest version
            r = requests.post(f'https://{self._url}/api/deposit/depositions/{deposition_id}/actions/newversion',
                              **self._default_params)
            r.raise_for_status()
            latest_deposition_draft_id = r.json()['id']

        # get full draft deposition json
        r = requests.get(
            f'https://{self._url}/api/deposit/depositions/{latest_deposition_draft_id}', **self._default_params)
        r.raise_for_status()
        draft = r.json()

        bucket_url = draft['links']['bucket']

        # remove files before adding new ones
        for file_entry in draft['files']:
            file_id = file_entry['id']
            r = requests.delete(
                f'https://{self._url}/api/deposit/depositions/{draft["id"]}/files/{file_id}', **self._default_params)
            r.raise_for_status()

        # bucket_id = zenodo.get_bucket_id(
        # deposition_id=deposition_id, zenodo_url=self._url, token=self._token)

        # assert bucket_url.endswith(bucket_id)

        # zenodo.create_new_zenodo_version(
        #     deposition_id=deposition_id, zenodo_url=self._url, token=self._token, deposit_metadata='exfor_metadata.json')
        # zenodo.upload_file(filename, bucket=bucket_id,
        #                    zenodo_url=self._url, token=self._token)

        # exit()

        # generate new version draft
        # r = requests.post(f"{self._url}/api/deposit/depositions/48819/actions/newversion",
        #                   **self._default_params)
        # r.raise_for_status()

        # add metadata to the deposition
        with open('exfor_metadata.json', 'r') as mf:
            metadata = json.load(mf)
        # update version information
        if version:
            metadata['version'] = version

        data = {'metadata': metadata}
        r = requests.put(f'https://{self._url}/api/deposit/depositions/{draft["id"]}',
                         data=json.dumps(data),
                         **self._default_params)
        r.raise_for_status()

        # # clean previous content
        # r = requests.get(
        #     f"{self._url}/api/deposit/depositions/{deposition_id}/files", **self._default_params)
        # for file in r.json():
        #     file_id = file['id']
        #     r = requests.delete(
        #         f"{self._url}/api/deposit/depositions/{deposition_id}/files/{file_id}", **self._default_params)
        #     assert r.status_code == 204

        # add content to zenodo bucket
        filepath = pathlib.Path(filename)
        ''' 
        The target URL is a combination of the bucket link with the desired filename
        separated by a slash.
        '''
        bucket_url = draft['links']['bucket']
        with open(filepath, "rb") as fp:
            r = requests.put(f"{bucket_url}/{filepath.name}",
                             data=fp,
                             params=self._default_params['params'],
                             verify=self._default_params['verify'])
        r.raise_for_status()

        # publish new version
        from http.client import RemoteDisconnected
        from urllib3.exceptions import ProtocolError
        try:
            r = requests.post(f'https://{self._url}/api/deposit/depositions/{draft["id"]}/actions/publish',
                          **self._default_params)
            r.raise_for_status()

        # handle strange zenodo behaviour where the post call to publish does not respond, but publishes the deposition
        except (requests.ConnectionError):
            r = requests.get(f'https://{self._url}/api/deposit/depositions/{draft["id"]}',
                             **self._default_params)
            deposit = r.json()
            if deposit['state'] != 'done' or deposit['submitted'] != True:
                raise ConnectionError(
                    f'Could not publish new version of deposit. Check manually at {deposit['links']['html']}')
        else:
            logging.info(f'Successfully published new version at {
                         r.json()['links']['html']}')

if __name__ == '__main__':

    parser = argparse.ArgumentParser('Register DOI')
    parser.add_argument("filename", type=str,
                        help="The file to register a doi for")
    parser.add_argument("-d", "--deposition_id", type=str,
                        help="An existing Zenodo deposition id to add the file to")
    parser.add_argument("-v", "--version", type=str,
                        help="The version number to register")
    parser.add_argument("-t", "--token", type=str,
                        help="The Zenodo API token. Can be also set via environment variable `self._token`")
    parser.add_argument("-u", "--url", type=str, choices=['sandbox.zenodo.org', 'zenodo.org'],
                        help="The Zenodo server to upload the file to. . Can be also set via environment variable `self._url`. Use the sandbox server for testing")
    args = parser.parse_args()

    filename = args.filename
    version = args.version
    deposition_id = args.deposition_id
    url = args.url or os.environ.get('self._url')
    token = args.token or os.environ.get('self._token')

    api = ZenodoAPI(url, token)
    api.register_doi(filename, version, deposition_id)
